from obspy.clients.fdsn import Client
from obspy import UTCDateTime
from datetime import timedelta
import pandas as pd
import argparse
import sys


def _cat2df(cat):
    cols = ['date', 'evla', 'evlo', 'evdp', 'mag', 'magtype']
    data = [[evt.origins[0].time, evt.origins[0].latitude, 
             evt.origins[0].longitude, evt.origins[0].depth*0.001,
             evt.magnitudes[0].mag, evt.magnitudes[0].magnitude_type] for evt in cat if evt.origins[0].depth is not None]
    return pd.DataFrame(data, columns=cols)


class Query():
    def __init__(self):
        self.client = Client("IRIS")

    def get_events(self, starttime=UTCDateTime(2000, 1, 1),
                   endtime=UTCDateTime.now(), **kwargs):
        events = self.client.get_events(starttime=starttime,
                                        endtime=endtime, **kwargs)
        self.events = _cat2df(events)

    def get_stations(self, includerestricted=False, **kwargs):
        self.stations = self.client.get_stations(includerestricted=includerestricted, **kwargs)

    def get_conti(self, starttime, endtime, hours=24):
        self.conti_time = []
        nowtime = starttime
        while nowtime < endtime:
            nexttime = nowtime + timedelta(hours=hours)
            self.conti_time.append([nowtime, nexttime])
            nowtime = nexttime


def get_events():
    parser = argparse.ArgumentParser(description="Get seismic events from IRIS WS")
    parser.add_argument('-b', help='Start time', type=str, default=None)
    parser.add_argument('-e', help='End time', type=str, default=None)
    parser.add_argument('-d', help='Radial geographic constraints with <lat>/<lon>/<minradius>/<maxradius>', type=str, default=None)
    parser.add_argument('-r', help='Box range with <lon1>/<lon2>/<lat1>/<lat2>', type=str, default=None)
    parser.add_argument('-m', help='Magnitude <minmagnitude>[/<maxmagnitude>]', type=str, default=None)
    parser.add_argument('-p', help='Focal depth <mindepth>[/<maxdepth>]', type=str, default=None)
    parser.add_argument('-c', help='Catalog', type=str, default=None)
    arg = parser.parse_args()
    args = {}
    if arg.c is not None:
        args['catalog'] = arg.c
    if arg.p is not None:
        try:
            values = [float(value) for value in arg.p.split('/')]
        except:
            raise ValueError('Error format with focal depth')
        if len(values) == 1:
            args['mindepth'] = values[0]
        elif len(values) == 2:
            args['mindepth'] = values[0]
            args['maxdepth'] = values[1]
        else:
            raise ValueError('Error format with focal depth')
    if arg.m is not None:
        try:
            values = [float(value) for value in arg.m.split('/')]
        except:
            raise ValueError('Error format with magnitude')
        if len(values) == 1:
            args['minmagnitude'] = values[0]
        elif len(values) == 2:
            args['minmagnitude'] = values[0]
            args['maxmagnitude'] = values[1]
        else:
            raise ValueError('Error format with focal depth')
    if arg.r is not None:
        try:
            values = [float(value) for value in arg.r.split('/')]
        except:
            raise ValueError('Error format with box range')
        if len(values) == 4:
            args['minlongitude'] = values[0]
            args['maxlongitude'] = values[1]
            args['minlatitude'] = values[2]
            args['maxlatitude'] = values[3]
        else:
            raise ValueError('Error format with box range')
    elif arg.d is not None:
        try:
            values = [float(value) for value in arg.d.split('/')]
        except:
            raise ValueError('Error format with Radial geographic constraints')
        if len(values) == 4:
            args['latitude'] = values[0]
            args['longitude'] = values[1]
            args['minradius'] = values[2]
            args['maxradius'] = values[3]
        else:
            raise ValueError('Error format with radial geographic constraints')
    else:
        pass
    if arg.b is not None:
        try:
            args['starttime'] = UTCDateTime(arg.b)
        except:
            raise ValueError('-b: Error format with time string')
    if arg.e is not None:
        try:
            args['endtime'] = UTCDateTime(arg.e)
        except:
            raise ValueError('-e: Error format with time string')
    if args == {}:
        parser.print_usage()
        sys.exit(1)
    query = Query()
    query.get_events(**args)
    for i, row in query.events.iterrows():
        print('{} {:.2f} {:.2f} {:.2f} {:.1f} {}'.format(
              row.date.isoformat(), row.evla, row.evlo, row.evdp, row.mag, row.magtype))


def get_stations():
    parser = argparse.ArgumentParser(description="Get stations from IRIS WS")
    parser.add_argument('-n', help='Network', type=str, default=None)
    parser.add_argument('-s', help='Station', type=str, default=None)
    parser.add_argument('-r', help='Box range with <lon1>/<lon2>/<lat1>/<lat2>', type=str, default=None)
    parser.add_argument('-d', help='Radial geographic constraints with <lat>/<lon>/<minradius>/<maxradius>', type=str, default=None)
    parser.add_argument('-b', help='Start time', type=str, default=None)
    parser.add_argument('-e', help='End time', type=str, default=None)
    parser.add_argument('-c', help='Channel', type=str, default=None)
    arg = parser.parse_args()
    args = {}
    if arg.n is not None:
        args['network'] = arg.n
    if arg.s is not None:
        args['station'] = arg.s
    if arg.r is not None:
        try:
            values = [float(value) for value in arg.r.split('/')]
        except:
            raise ValueError('Error format with box range')
        if len(values) == 4:
            args['minlongitude'] = values[0]
            args['maxlongitude'] = values[1]
            args['minlatitude'] = values[2]
            args['maxlatitude'] = values[3]
        else:
            raise ValueError('Error format with box range')
    elif arg.d is not None:
        try:
            values = [float(value) for value in arg.d.split('/')]
        except:
            raise ValueError('Error format with Radial geographic constraints')
        args['latitude'] = values[0]
        args['longitude'] = values[1]
        args['minradius'] = values[2]
        args['maxradius'] = values[3]
    else:
        pass
    if arg.b is not None:
        try:
            args['starttime'] = UTCDateTime(arg.b)
        except:
            raise ValueError('-b: Error format with time string')
    if arg.e is not None:
        try:
            args['endtime'] = UTCDateTime(arg.e)
        except:
            raise ValueError('-e: Error format with time string')
    if arg.c is not None:
        args['channel'] = arg.c
    if args == {}:
        parser.print_usage()
        sys.exit(1)
    query = Query()
    query.get_stations(**args)
    for net in query.stations:
        for sta in net:
            print('{} {} {:.4f} {:.4f} {} {} {}'.format(net.code, sta.code,
                  sta.latitude, sta.longitude, sta.start_date, sta.end_date,
                  sta.restricted_status))


if __name__ == "__main__":
    query = Query()
    # query.get_events(minmagnitude=7, catalog='GCMT')
    # query.get_stations(network='3J')
    # query.get_stations(network='3J', minlatitude=20,
    #                    minlongitude=97, maxlatitude=40,
    #                    maxlongitude=110)
    query.get_conti(UTCDateTime(2011,1,1), UTCDateTime(2011,2,1))
    print(query.conti_time)
    # print(query.stations)



from bqmail.query import Query
from obspy import UTCDateTime
from obspy.taup import TauPyModel
from obspy.clients.iris import Client
import smtplib
from email.mime.text import MIMEText
from email.header import Header 
from .distaz import distaz
model = TauPyModel()
cld = Client()


def connectsmtp(server, port, sender, password):
    smtpObj = smtplib.SMTP_SSL(server, port)
    smtpObj.login(sender, password)
    return smtpObj


def loginmail(sender, server='localhost', password='', port=465, test_num=5):
    test_it = 1
    while test_it <= test_num:
        if test_it > 1:
            print('Try to link to {} in {} times'.format(server, test_it))
        try:
            smtpObj = connectsmtp(server, port, sender, password)
        except:
            test_it += 1
            continue
        else:
            break
    if test_it > test_num:
        raise ConnectionError("Error in linking {}".format(server))
    else:
        return smtpObj


def sendmail(smtpObj, sender, contents,
             recipient='breq_fast@iris.washington.edu'):
    msg = MIMEText(contents, 'text')
    msg['Subject'] = Header('batch request seismic data', 'utf-8')
    msg['From'] = Header(sender)
    msg['To'] = Header(recipient)
    try:
        smtpObj.sendmail(sender, recipient, msg.as_string())
        return True
    except Exception as e:
        print('Error in sending mail: {}'.format(e))
        return False


def generatemsg(username, inst, mailname, media):
    msg = ''
    msg += '.NAME '+username+'\n'
    msg += '.INST '+inst+'\n'
    msg += '.MAIL\n'
    msg += '.EMAIL '+mailname+'\n'
    msg += '.PHONE\n'
    msg += '.FAX\n'
    msg += '.MEDIA '+media+'\n'
    msg += '.ALTERNATE MEDIA '+media+'\n'
    msg += '.ALTERNATE MEDIA '+media+'\n'
    return msg


class BQMail():
    def __init__(self, mailname, username='bqmail',
                 inst='', media='Electronic (FTP)',
                 server='localhost', password=''):
        self.query = Query()
        self.mailname = mailname
        self.server = server
        self.password = password
        self.username = username
        self.inst = inst
        self.media = media
        self.msgs = ''
        self.label = ''
        self.header = generatemsg(username, inst, mailname, media)

    def query_stations(self, **kwargs):
        self.query.get_stations(**kwargs)

    def query_events(self, **kwargs):
        self.query.get_events(**kwargs)

    def send_mail(self, arrange='events', write_mail=False, **kwargs):
        if arrange == 'events':
            self.event_mail(**kwargs)
        elif arrange == 'continue':
            self.conti_mail(**kwargs)
        else:
            raise ValueError('variable arrange must be in \'events\' and \'continue\'')
        smtpobj = loginmail(self.mailname, server=self.server,
                            password=self.password)
        succ = sendmail(smtpobj, self.mailname, self.msgs)
        if succ:
            print('successfully send {}'.format(self.label))
        else:
            print('Error in sending {}'.format(self.label))
        if write_mail:
            with open('msg.{}'.format(self.label), 'w') as f:
                f.write(self.msgs)

    def conti_mail(self, starttime=UTCDateTime(2000, 1, 1),
                   endtime=UTCDateTime.now(), time_val_in_hours=24,
                   channel='BH?', location=''):
        self.query.get_conti(starttime, endtime, hours=time_val_in_hours)
        self.msgs = self.header
        self.label = '{}_{}'.format(starttime.strftime('%Y.%m.%d'),
                                    endtime.strftime('%Y.%m.%d'))
        self.msgs += '.LABEL {}\n.END\n'.format(self.label)
        for sdt, edt in self.query.conti_time:
            for net in self.query.stations:
                for sta in net:
                    self.msgs += '{} {} {} {} 1 {} {}\n'.format(
                                 sta.code, net.code, sdt.strftime('%Y %m %d %H %M %S'), 
                                 edt.strftime('%Y %m %d %H %M %S'), channel, location)

    def event_mail(self, time_before=0, time_after=1000,
                   mark='o', channel='BH?', location=''):
        self.msgs = self.header
        self.label = 'Evts_{}'.format(UTCDateTime.now().strftime('%Y.%m.%dT%H%M%S'))
        self.msgs += '.LABEL {}\n.END\n'.format(self.label)
        for net in self.query.stations:
            for sta in net:
                if mark == 'o':
                    for i, evt in self.query.events.iterrows():
                        b_time = (evt['date'] + time_before).strftime('%Y %m %d %H %M %S')
                        e_time = (evt['date'] + time_after).strftime('%Y %m %d %H %M %S')
                        self.msgs += '{} {} {} {} 1 {} {}\n'.format(
                                     sta.code, net.code, b_time, e_time,
                                     channel, location)
                else:
                    for i, evt in self.query.events.iterrows():
                        ttime = self.get_ttime(evt, sta, phase=mark)
                        if ttime is None:
                            continue
                        else:
                            b_time = (evt['date'] + ttime + time_before).strftime('%Y %m %d %H %M %S')
                            e_time = (evt['date'] + ttime + time_after).strftime('%Y %m %d %H %M %S')
                            self.msgs += '{} {} {} {} 1 {} {}\n'.format(
                                         sta.code, net.code, b_time, e_time,
                                         channel, location)

    def get_ttime(self, evt, obspy_station, phase='P'):
        da = distaz(obspy_station.latitude, obspy_station.longitude,
                    evt.evla, evt.evlo)
        arr = model.get_travel_times(evt.evdp, da.delta, phase_list=[phase])
        if len(arr) == 0:
            return None
        else:
            return arr[0].time




#!/usr/bin/env python
from setuptools import find_packages, setup
packages = find_packages()

with open("README.md", "r") as fh:
    long_description = fh.read()

VERSION = "2.1.1"
setup(name='bqmail',
      version=VERSION,
      author='Mijian Xu',
      author_email='gomijianxu@gmail.com',
      url='https://github.com/xumi1993/bqmail',
      long_description=long_description,
      long_description_content_type="text/markdown",
      license='MIT',
      packages=find_packages(),
      package_dir={'bqmail': 'bqmail'},
      package_data={'': ['data/*']},
      install_requires=['obspy', 'pandas'],
      entry_points={'console_scripts': ['get_stations=bqmail.query:get_stations',
                                        'get_events=bqmail.query:get_events']},
      include_package_data=True,
      zip_safe=False,
      classifiers=['Programming Language :: Python',
                   'Programming Language :: Python :: 3.6',
                   'Programming Language :: Python :: 3.7',
                   'Programming Language :: Python :: 3.8']
      )
